## Distributed as part of the supplementary material for the manuscript
## "Jeffreys-prior penalty, finiteness and shrinkage in binomial-response generalized linear models"
##
## Authors: Ioannis Kosmidis, David Firth
## Date: 19 March 2020
## Licence: GPL 2 or greater
## NOT A POLISHED PIECE OF PUBLIC-USE SOFTWARE!  Provided "as is".
## NO WARRANTY OF FITNESS FOR ANY PURPOSE!

library("brglm2")
library("qvcalc")
library("ggplot2")
source("nba-1415-functions.R")

## Load the data
## Data is all outcomes in the regular season of 2014-2015 downloaded from
## https://www.basketball-reference.com/leagues/NBA_2015_games-october.html
nba_1415 <- read.csv("nba-1415-regular-season.csv")

## Prepare data into winners and losers, and format timestamps
nba_1415_competitions <- with(nba_1415, {
    data.frame(time = as.POSIXct(paste0(date, ", ", time), format = "%a, %b %d, %Y, %I:%M %p", tz = "EST"),
               winner = ifelse(home_points > away_points, as.character(home), as.character(away)),
               winner_points = ifelse(home_points > away_points, home_points, away_points),
               loser = ifelse(home_points < away_points, as.character(home), as.character(away)),
               loser_points = ifelse(home_points < away_points, home_points, away_points),
               home = home,
               away = away)
})

## Fitting a Bradley-Terry model on all competitions before 03 Dec 2014, which is the first time
## Philadelphia won a match against Minnesota Tomberwolves, after 17 loses in a row!
nba_phil <- subset(nba_1415_competitions, as.Date(time, tz = "EST") < as.Date("2014-12-03", tz = "EST"))
nba_phil_BT <- prepare_BT_df(nba_phil)
## Setup BT_formula with San Antonio Spurs as reference (entering it last)
BT_formula <- y ~ -1 + . - San.Antonio.Spurs + San.Antonio.Spurs

fit_ml <- glm(BT_formula, data = nba_phil_BT, family = binomial(logit))
qv_ml <- qvfit(fit_ml)

## Test for separation; if maximum likelihood is used, the ability of
## Philadelphia 76ers is estimated as -Inf, indicating that they will
## always lose, which is clearly nonsense.
update(fit_ml, method = "detect_separation")

## A reduced-bias fit returns finite estmimates
fit_br <- update(fit_ml, method = "brglm_fit")
qv_br <- qvfit(fit_br)

## Confidence intervals for contrasts
quant <- qnorm(0.975)
vcov_results <- rbind(with(qv_ml, data.frame(low = qvframe$estimate - quant * qvframe$SE,
                                             upp = qvframe$estimate + quant * qvframe$SE,
                                             estimate = qvframe$estimate,
                                             parameter = gsub("\\.", " ", rownames(qvframe)),
                                             method = "ML")),
                      with(qv_br, data.frame(low = qvframe$estimate - quant * qvframe$SE,
                                             upp = qvframe$estimate + quant * qvframe$SE,
                                             estimate = qvframe$estimate,
                                             parameter = gsub("\\.", " ", rownames(qvframe)),
                                             method = "RB")))

## Figure 1
g1 <- ggplot(vcov_results) +
    geom_point(aes(x = parameter, y = estimate)) +
    geom_segment(aes(x = parameter, xend = parameter, y = low, yend = upp)) +
    facet_grid(method ~ .) +
    coord_cartesian(ylim = c(-7, 7)) +
    labs(x = "team", y = "estimated ablility") +
    theme_minimal() +
    theme(axis.text.x = element_text(angle = 60, hjust = 1, size = 8))

g2 <- ggplot(data.frame(ml = subset(vcov_results, method == "ML")$estimate,
                        br = subset(vcov_results, method == "RB")$estimate)) +
    geom_point(aes(ml, br)) +
    geom_abline(aes(intercept = 0, slope = 1)) +
    coord_cartesian(xlim = c(-2, 2), ylim = c(-2, 2)) +
    labs(x = "ML", y = "RB") +
    theme_minimal()
print(g1)
print(g2)

## Table S1 in the supplementary material
source("jeffreys-MPL.R")
nba_phil_BT2 <- with(nba_phil, {
    data.frame(y = as.numeric(winner == home), ## home win
               home = home,
               away = away)
})
n_games <- nrow(nba_phil_BT2)
teams <- with(nba_phil_BT2, unique(c(levels(away), levels(home))))
n_teams <-  length(teams)
X <- matrix(0, nrow = n_games, ncol = n_teams)
colnames(X) <- teams
for (i in seq.int(nrow(X))) {
    X[i, as.character(nba_phil_BT2[i, "home"])] <- 1
    X[i, as.character(nba_phil_BT2[i, "away"])] <- -1
}
nba_phil_BT2 <- data.frame(y = nba_phil_BT2$y, X)

out <- JeffreysMPL(y = nba_phil_BT2$y, X = X[, -which(colnames(X) %in% "San Antonio Spurs")],
                   adj = 0.01,
                   a = 0.5, epsilon = 1e-10)

phil_games <- which(nba_phil$home == "Philadelphia 76ers" | nba_phil$away == "Philadelphia 76ers")[1:6]
n_phil_games <- length(phil_games)
home_phil <- nba_phil$home[phil_games]
away_phil <- nba_phil$away[phil_games]

iter <- 6
resp_mat <- t(out$adjusted_responses[phil_games, seq.int(iter)])
tot_mat <- t(out$adjusted_totals[phil_games, seq.int(iter)])
dimnames(resp_mat) <- dimnames(tot_mat) <- list(seq.int(iter), paste(home_phil, "vs", away_phil))
round(resp_mat, 3)
round(tot_mat, 3)

## Figure 3
consts <- unique(c(0.5, seq(0.01, 5, length = 100)))
abilities <- NULL
for (j in seq_along(consts)) {
    .const <- consts[j]
    coefs <- JeffreysMPL(y = nba_phil_BT2$y, X = X[, -which(colnames(X) %in% "San Antonio Spurs")],
                         adj = 0.01,
                         a = consts[j], epsilon = 1e-10)$coef
    coefs <- c(0, coefs)
    names(coefs)[1] <- "San.Antonio.Spurs"
    abilities <- rbind(abilities,
                       data.frame(a = .const, ability = coefs, team = gsub("\\.", " ", names(coefs))))
}

ggplot(abilities) +
    geom_vline(aes(xintercept = 0.5), lty = 3, col = "gray") +
    geom_hline(aes(yintercept = 0), lty = 2, col = "gray") +
    geom_line(aes(x = a, y = ability)) +
    facet_wrap(team ~ ., ncol = 6) +
    labs(x = "a", y = "ability contrast") +
    theme_minimal()



## library("gridExtra")
## pdf("abilities.pdf", width = 10, height = 4)
## grid.arrange(g1, g2, nrow = 1, widths = c(3, 2))
## dev.off()
